# FreePWING parser generator
# Tamito KAJIYAMA <14 September 2001>

import string

def value(val):
    if type(val) == type(""):
        val = string.replace(val, "\r", "")
        val = string.replace(val, "\n", "")
        return '"%s"' % string.replace(val, '"', '\\"')
    return str(val)

class Value:
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return self.value

class Variable:
    serial = 0
    def __init__(self, value=None):
        Variable.serial = Variable.serial + 1
        self.name = "$var%d" % Variable.serial
        if value is not None:
            self.set(value)
    def set(self, value):
        print "%s = %s;" % (self.name, value)
    def __str__(self):
        return self.name

class FreePWING:
    def __init__(self, name):
        self.name = name
    def _execute(self, command, success=1):
        print '$fpw%s->%s || die $fpw%s->error_message() . "\\n";' % (
            self.name, command, self.name)
    def _value(self, name):
        return Value("$fpw%s->%s()" % (self.name, name))
    def open(self):
        self._execute("open()")
    def close(self):
        self._execute("close()")
    def file_name(self):
        return self._value("file_name")
    def entry_count(self):
        return self._value("entry_count")

class Word2(FreePWING):
    def __init__(self):
        FreePWING.__init__(self, "word2")
    def add_entry(self, word, heading_pos, text_pos):
        self._execute(
            'add_entry(%s, %s, $heading_file_name, %s, $text_file_name)' % (
                value(word), value(heading_pos), value(text_pos)))

class Heading(FreePWING):
    def __init__(self):
        FreePWING.__init__(self, "heading")
    def new_entry(self):
        self._execute("new_entry()")
    def add_text(self, text):
        self._execute('add_text(%s)' % value(text))
    def add_half_user_character(self, name):
        self._execute('add_half_user_character(%s)' % value(name))
    def add_full_user_character(self, name):
        self._execute('add_full_user_character(%s)' % value(name))
    def add_entry_tag(self, name):
        self._execute('add_entry_tag(%s)' % value(name))
    def add_subscript_start(self):
        self._execute('add_subscript_start()')
    def add_subscript_end(self):
        self._execute('add_subscript_end()')
    def add_superscript_start(self):
        self._execute('add_superscript_start()')
    def add_superscript_end(self):
        self._execute('add_superscript_end()')
    def add_nowrap_start(self):
        self._execute('add_nowrap_start()')
    def add_nowrap_end(self):
        self._execute('add_nowrap_end()')
    def add_emphasis_start(self):
        self._execute('add_emphasis_start()')
    def add_emphasis_end(self):
        self._execute('add_emphasis_end()')
    def add_font_start(self, name):
        self._execute('add_font_start(%s)' % value(name))
    def add_font_end(self):
        self._execute('add_font_end()')
    def reference_file_name(self):
        return self._value("reference_file_name")
    def tag_file_name(self):
        return self._value("tag_file_name")
    def context_position(self):
        return self._value("context_position")
    def entry_position (self):
        return self._value("entry_position")
    def context_count(self):
        return self._value("context_count")
    def entry_count (self):
        return self._value("entry_count")
    def total_entry_count(self):
        return self._value("total_entry_count")

class Text(Heading):
    def __init__(self):
        FreePWING.__init__(self, "text")
    def new_context(self):
        self._execute("new_context()")
    def add_tag(self, name):
        self._execute('add_tag(%s)' % value(name))
    def add_keyword_start(self):
        self._execute("add_keyword_start()")
    def add_keyword_end(self):
        self._execute("add_keyword_end()")
    def add_reference_start(self):
        self._execute("add_reference_start()")
    def add_reference_end(self, name):
        self._execute('add_reference_end(%s)' % value(name))
    def add_newline(self):
        self._execute("add_newline()")
    def add_indent_level(self, level):
        self._execute('add_indent_level(%s)' % value(level))

class Menu(Text):
    def __init__(self):
        FreePWING.__init__(self, "menu")

class Copyright(Text):
    def __init__(self):
        FreePWING.__init__(self, "copyright")

def initialize_fpwparser(*args):
    print "use FreePWING::FPWUtils::FPWParser;"
    buffer = []
    for obj in args:
        buffer.append("'%s' => \\$fpw%s" % (obj.name, obj.name))
    print "initialize_fpwparser(%s);" % string.join(buffer, ", ")

def finalize_fpwparser(*args):
    buffer = []
    for obj in args:
        buffer.append("'%s' => \\$fpw%s" % (obj.name, obj.name))
    print "finalize_fpwparser(%s);" % string.join(buffer, ", ")
